=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-01-16 14:05:41 +0000
+++ bzrlib/branch.py	2007-01-16 14:26:41 +0000
@@ -1262,25 +1262,30 @@
         finally:
             source.unlock()
 
+    def _get_parent_location(self):
+        _locs = ['parent', 'pull', 'x-pull']
+        for l in _locs:
+            try:
+                return self.control_files.get(l).read().strip('\n')
+            except NoSuchFile:
+                pass
+        return None
+
     def get_parent(self):
         """See Branch.get_parent."""
 
-        _locs = ['parent', 'pull', 'x-pull']
         assert self.base[-1] == '/'
-        for l in _locs:
-            try:
-                parent = self.control_files.get(l).read().strip('\n')
-            except NoSuchFile:
-                continue
-            # This is an old-format absolute path to a local branch
-            # turn it into a url
-            if parent.startswith('/'):
-                parent = urlutils.local_path_to_url(parent.decode('utf8'))
-            try:
-                return urlutils.join(self.base[:-1], parent)
-            except errors.InvalidURLJoin, e:
-                raise errors.InaccessibleParent(parent, self.base)
-        return None
+        parent = self._get_parent_location()
+        if parent is None:
+            return parent
+        # This is an old-format absolute path to a local branch
+        # turn it into a url
+        if parent.startswith('/'):
+            parent = urlutils.local_path_to_url(parent.decode('utf8'))
+        try:
+            return urlutils.join(self.base[:-1], parent)
+        except errors.InvalidURLJoin, e:
+            raise errors.InaccessibleParent(parent, self.base)
 
     def get_push_location(self):
         """See Branch.get_push_location."""
@@ -1301,9 +1306,7 @@
         # FIXUP this and get_parent in a future branch format bump:
         # read and rewrite the file, and have the new format code read
         # using .get not .get_utf8. RBC 20060125
-        if url is None:
-            self.control_files._transport.delete('parent')
-        else:
+        if url is not None:
             if isinstance(url, unicode):
                 try: 
                     url = url.encode('ascii')
@@ -1313,6 +1316,13 @@
                         "use bzrlib.urlutils.escape")
                     
             url = urlutils.relative_url(self.base, url)
+        self._set_parent_location(url)
+
+    def _set_parent_location(self, url):
+        if url is None:
+            self.control_files._transport.delete('parent')
+        else:
+            assert isinstance(url, str)
             self.control_files.put('parent', StringIO(url + '\n'))
 
     @deprecated_function(zero_nine)
@@ -1488,29 +1498,20 @@
             self.set_last_revision(history[-1])
 
     @needs_write_lock
-    def set_parent(self, url):
+    def _set_parent_location(self, url):
         """Set the parent branch"""
-        if isinstance(url, unicode):
-            try: 
-                url = url.encode('ascii')
-            except UnicodeEncodeError:
-                raise bzrlib.errors.InvalidURL(url,
-                    "Urls must be 7-bit ascii, "
-                    "use bzrlib.urlutils.escape")
-                
+        if url is None:
+            url = ''
         url = urlutils.relative_url(self.base, url)
         self.get_config().set_user_option('parent_location', url)
 
     @needs_read_lock
-    def get_parent(self):
+    def _get_parent_location(self):
         """Set the parent branch"""
         parent = self.get_config().get_user_option('parent_location')
-        if parent is None:
-            return parent
-        try:
-            return urlutils.join(self.base[:-1], parent)
-        except errors.InvalidURLJoin, e:
-            raise errors.InaccessibleParent(parent, self.base)
+        if parent == '':
+            parent = None
+        return parent
 
     def set_push_location(self, location):
         """See Branch.set_push_location."""

=== modified file 'bzrlib/tests/branch_implementations/test_parent.py'
--- bzrlib/tests/branch_implementations/test_parent.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/branch_implementations/test_parent.py	2007-01-16 14:26:41 +0000
@@ -42,7 +42,7 @@
         url = 'http://bazaar-vcs.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEqual(url, b.get_parent())
-        self.assertEqual(url, b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual(url, b._get_parent_location())
 
         b.set_parent(None)
         self.assertEqual(None, b.get_parent())
@@ -52,15 +52,13 @@
         self.assertEqual(local_path_to_url('../other_branch'), b.get_parent())
         path = local_path_to_url('../yanb')
         b.set_parent(path)
-        self.assertEqual('../yanb',
-            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual('../yanb', b._get_parent_location())
         self.assertEqual(path, b.get_parent())
 
 
         self.assertRaises(bzrlib.errors.InvalidURL, b.set_parent, u'\xb5')
         b.set_parent(escape(u'\xb5'))
-        self.assertEqual('%C2%B5',
-            b.control_files.get('parent').read().strip('\n'))
+        self.assertEqual('%C2%B5', b._get_parent_location())
 
         self.assertEqual(b.base + '%C2%B5', b.get_parent())
 
@@ -70,7 +68,7 @@
             #       paths as well? Nobody has complained about it.
             pass
         else:
-            b.control_files.put('parent', cStringIO.StringIO('/local/abs/path'))
+            b._set_parent_location('/local/abs/path')
             self.assertEqual('file:///local/abs/path', b.get_parent())
 
     def test_get_invalid_parent(self):
@@ -82,7 +80,7 @@
         # Force the relative path to be something invalid
         # This should attempt to go outside the filesystem
         path = ('../'*(n_dirs+5)) + 'foo'
-        b.control_files.put('parent', cStringIO.StringIO(path))
+        b._set_parent_location(path)
 
         # With an invalid branch parent, just return None
         self.assertRaises(bzrlib.errors.InaccessibleParent, b.get_parent)

